/**
 * Module dependencies.
 */

//require.paths.unshift('models/')
var express  = require('express'),
    mongoose = require('mongoose'),
    io       = require('socket.io'),
    models   = require('./models'),
    db,
    Metric;

var app = module.exports = express.createServer();

// Configuration

app.configure(function(){
    app.set('views', __dirname + '/views');
    app.set('view engine', 'jade');
    app.set('db-uri', 'mongodb://localhost/metrics');
    app.use(express.bodyParser());
    app.use(express.methodOverride());
    app.use(app.router);
    app.use(express.static(__dirname + '/public'));
    app.use(express.logger());
    app.use(express.errorHandler({ dumpExceptions: true, showStack: true })); 
});

// Import des models dans le scope courant (?)

models.defineModels(mongoose, function() {
    app.Metric = Metric = mongoose.model('Metric');
    db = mongoose.connect(app.set('db-uri'));
})


// Routes

app.get('/', function(req, res) {
    res.render('index', {
        title: 'aumepaje'
    });
});

app.get('/metric/:host/:name.:format?', function(req, res) {
    // XXX trouver comment sort()er/limit()er via mongoose...
    // (au lieu de connement tout recuperer puis filtrer en js)
    Metric.find({host: req.params.host, name: req.params.name}, 
        function(err, m) {
            var last = m.pop()
            res.render('metrics/single.jade', {
                locals: { m: last, title: last.host + ' - ' + last.name }
            });
    });
});

app.get('/metric/all.:format?', function(req, res) {
    // XXX trouver comment faire appliquer un mapreduce via mongoose
    // Car ceci retourne toutes les valeurs (or on voudrait filtrer
    // pour obtenir les dernieres valeur de chaque hote+compteur).
    Metric.find({}, (function(err, metrics) {
        switch (req.params.format) { 
            case 'json':
                res.send(metrics.map(function(m) {
                    return m.toObject();
                }));
                break;
            default:
                metrics = metrics.map(function(m) {
                    return { name: m.name, host: m.host, value: m.value};
                });
                res.render('metrics/all.jade', {
                    locals: { metrics: metrics, title: "foo" }
                });
        }
    }));
});

app.post('/metric/:metric.:format?', function(req, res) {
    // juste pour tester la notification temps reel aux navigateurs
    // une vraie appli devrais selectioner les clients souscrits au canal...
    io.broadcast(req.body.value);

    // puis stockage dans mongodb (via mongoose)
    new Metric({
        host: req.body.host,
        name: req.body.name,
        value: req.body.value}).
    save(function() { res.send("OK\n"); } );
});


// launch time

if (!module.parent) { //check if the app is running as the top-level module
    app.listen(3000);
    console.log("server listening on port %d", app.address().port);

    var io = io.listen(app, { flashPolicyServer: false });

    io.on('connection', function(client){

        client.on('message', function(message){
            console.log("socket: %s", message);
            client.broadcast(message.text);
        });

    });
}

